@@ -1,4 +1,3 @@
-
 # Licensed to the Apache Software Foundation (ASF) under one
 # or more contributor license agreements.  See the NOTICE file
 # distributed with this work for additional information
@@ -19,7 +18,7 @@ import re
 import zlib
 
 import cql
-from cql.marshal import prepare_inline, prepare_query, PreparedQuery
+from cql.query import prepare_inline, prepare_query, PreparedQuery
 from cql.decoders import SchemaDecoder
 from cql.cassandra.ttypes import (
     Compression,
@@ -43,11 +42,13 @@ class Cursor:
     _ddl_re = re.compile("\s*(CREATE|ALTER|DROP)\s+",
                          re.IGNORECASE | re.MULTILINE)
     supports_prepared_queries = False
+    supports_column_types = True
     supports_name_info = True
 
     def __init__(self, parent_connection):
         self.open_socket = True
         self._connection = parent_connection
+        self.cql_major_version = parent_connection.cql_major_version
 
         # A list of 7-tuples corresponding to the column metadata for the
         # current row (populated on execute() and on fetchone()):
@@ -92,6 +93,8 @@ class Cursor:
         return self.compress_query_text(prepared_q_text)
 
     def prepare_query(self, query, paramtypes=None):
+        if isinstance(query, unicode):
+            raise ValueError("CQL query must be bytes, not unicode")
         prepared_q_text, paramnames = prepare_query(query)
         compressed_q, compression = self.compress_query_text(prepared_q_text)
         presult = self._connection.client.prepare_cql_query(compressed_q, compression)
@@ -108,8 +111,11 @@ class Cursor:
         self.rowcount = 0
         self.description = None
         self.name_info = None
+        self.column_types = None
 
     def execute(self, cql_query, params={}, decoder=None):
+        if isinstance(cql_query, unicode):
+            raise ValueError("CQL query must be bytes, not unicode")
         self.pre_execution_setup()
 
         prepared_q, compress = self.prepare_inline(cql_query, params)
@@ -149,7 +155,7 @@ class Cursor:
             self.rs_idx = 0
             self.rowcount = len(self.result)
             if self.result:
-                self.description, self.name_info = self.decoder.decode_metadata(self.result[0])
+                self.get_metadata_info(self.result[0])
         elif response.type == CqlResultType.INT:
             self.result = [(response.num,)]
             self.rs_idx = 0
@@ -169,6 +175,10 @@ class Cursor:
         # 'Return values are not defined.'
         return True
 
+    def get_metadata_info(self, row):
+        self.description, self.name_info, self.column_types = \
+                self.decoder.decode_metadata_and_types(row)
+
     def executemany(self, operation_list, argslist):
         self.__checksock()
         opssize = len(operation_list)
@@ -192,19 +202,21 @@ class Cursor:
         if self.description is _COUNT_DESCRIPTION:
             return row
         else:
-            self.description, self.name_info = self.decoder.decode_metadata(row)
-            return self.decoder.decode_row(row)
+            if self.cql_major_version < 3:
+                # (don't bother redecoding descriptions or names otherwise)
+                self.get_metadata_info(row)
+            return self.decoder.decode_row(row, self.column_types)
 
     def fetchmany(self, size=None):
         self.__checksock()
         if size is None:
             size = self.arraysize
-        # we avoid leveraging fetchone here to avoid calling decode_metadata unnecessarily
+        # we avoid leveraging fetchone here to avoid decoding metadata unnecessarily
         L = []
         while len(L) < size and self.rs_idx < len(self.result):
             row = self.result[self.rs_idx]
             self.rs_idx += 1
-            L.append(self.decoder.decode_row(row))
+            L.append(self.decoder.decode_row(row, self.column_types))
         return L
 
     def fetchall(self):
