@@ -40,6 +40,7 @@ sys.path.append(join(abspath(dirname(__file__)), '..'))
 
 import cql
 from cql.cassandra import Cassandra
+from cql.cqltypes import AsciiType, UUIDType
 
 def get_thrift_client(host=TEST_HOST, port=TEST_PORT):
     socket = TSocket.TSocket(host, port)
@@ -184,6 +185,9 @@ class TestCql(unittest.TestCase):
     def get_partitioner(self):
         return thrift_client.describe_partitioner()
 
+    def assertIsSubclass(self, class_a, class_b):
+        assert issubclass(class_a, class_b), '%r is not a subclass of %r' % (class_a, class_b)
+
         
     def test_select_simple(self):
         "single-row named column queries"
@@ -646,7 +650,7 @@ class TestCql(unittest.TestCase):
                           "CREATE INDEX ON CreateIndex1 (stuff)")
 
     def test_drop_indexes(self):
-        "droping indexes on columns"
+        "dropping indexes on columns"
         cursor = self.cursor
         ksname = self.make_keyspace_name('DropIndexTests')
         cursor.execute("""CREATE KEYSPACE :ks WITH strategy_options:replication_factor = '1'
@@ -685,8 +689,10 @@ class TestCql(unittest.TestCase):
             SELECT KEY, '%s' FROM StandardTimeUUID WHERE KEY = 'uuidtest'
         """ % str(timeuuid))
         self.assertEqual(len(cursor.name_info), 2)
-        self.assertEqual(cursor.name_info[0], ('KEY', 'AsciiType'))
-        self.assertEqual(cursor.name_info[1], (timeuuid.bytes, 'UUIDType'))
+        self.assertEqual(cursor.name_info[0][0], 'KEY')
+        self.assertIsSubclass(cursor.name_info[0][1], AsciiType)
+        self.assertEqual(cursor.name_info[1][0], timeuuid.bytes)
+        self.assertIsSubclass(cursor.name_info[1][1], UUIDType)
 
     def test_time_uuid(self):
         "store and retrieve time-based (type 1) uuids"
@@ -1471,3 +1477,6 @@ class TestCql(unittest.TestCase):
         cursor.execute("TRUNCATE StandardString1")
         cursor.execute("SELECT * FROM StandardString1")
         assert cursor.rowcount == 0, "expected zero results, got %d" % cursor.rowcount
+
+    def test_reject_unicode(self):
+        self.assertRaises(ValueError, self.cursor.execute, u'select * from system.schema_keyspaces')
